#version 150

/*
 * Created by Onnowhere (https://github.com/onnowhere)
 * Utility functions for Minecraft core vertex shaders
 */

#define LIGHT0_DIRECTION vec3(0.2, 1.0, -0.7) // Default light 0 direction everywhere except in inventory
#define LIGHT1_DIRECTION vec3(-0.2, 1.0, 0.7) // Default light 1 direction everywhere except in nether and inventory
#define TINY 8e-8
#define PI 3.14159265359

/*
 * Returns the FOV in degrees
 * Calculates using the fact that top/near = tan(theta / 2)
 */
float getFOV(mat4 ProjMat) {
    return atan(1.0, ProjMat[1][1]) * 114.591559;
}

/*
 * Returns if rendering in a GUI
 * In the GUI, near is 1000 and far is 3000, so -(far+near)/(far-near) = -2.0
 */
bool isGUI(mat4 ProjMat) {
    return ProjMat[3][2] == -2.0;
}

/*
 * Returns if the vec4 is the same as the rgb color
 */
bool isSameColor3(vec4 color,float r,float g,float b) {
    return color.xyz == vec3(r,g,b);
}

/*
 * Returns if the vec4 is the same as the rgb color using vec4
 */
bool isSameColor4(vec4 color,float r,float g,float b) {
    color.a = 1;
    return color == vec4(r/255,g/255,b/255,1);
}

/*
 * Returns The vec4 as the rgb color using vec4
 */
vec4 colorByRgb4(vec4 color,float r,float g,float b) {
    return vec4(r/255,g/255,b/255,1);
}
/*
 * Returns The vec4 as the argb color using vec4
 */
vec4 colorByArgb4(vec4 color,float r,float g,float b,float a) {
    if(a>1){
        a = (a/100);
    }
    return vec4(r/255,g/255,b/255,a);
}
/*
 * Returns The vec4 as the rgb color using vec4
 */
vec3 colorByRgb3(vec4 color,float r,float g,float b) {
    return vec3(r,g,b);
}

/*
 * Returns if rendering in the main menu background panorama
 * Checks the far clipping plane value so this should only be used with position_tex_color
 */
bool isPanorama(mat4 ProjMat) {
    float far = ProjMat[3][2] / (ProjMat[2][2] + 1);
    return far < 9.99996 && far > 9.99995;
}

/*
 * Returns if rendering in the nether given light directions
 * In the nether, the light directions are parallel but in opposite directions
 */
bool isNether(vec3 light0, vec3 light1) {
    return light0 == -light1;
}

/*
 * Returns camera to world space matrix given light directions
 * Creates matrix by comparing world space light directions to camera space light directions
 */
mat3 getWorldMat(vec3 light0, vec3 light1) {
    if (isNether(light0, light1)) {
        // Cannot determine matrix in the nether due to parallel light directions
        return mat3(1.0);
    }
    mat3 V = mat3(normalize(LIGHT0_DIRECTION), normalize(LIGHT1_DIRECTION), normalize(cross(LIGHT0_DIRECTION, LIGHT1_DIRECTION)));
    mat3 W = mat3(normalize(light0), normalize(light1), normalize(cross(light0, light1)));
    return W * inverse(V);
}

mat3 getInvWorldMat(vec3 light0, vec3 light1) {
    return transpose(getWorldMat(light0, light1));
}